#include "perceptron.h"

int sign(double x) {
  if (x > 0) {
    return 1;
  }
  return -1;
}

int predict(const Vector3D& W, const Vector3D& x) {
  return sign(W*x);
}

void trainPerceptron(const std::vector<Vector3D>& X, const std::vector<int>& T, Vector3D& W, double lambda) {
  int i = 0;
  for (xi : X) {
    int u = predict(W, xi);
    if (u != T[i]) {
      W = W + xi * lambda * (T[i] - u);
    }
    ++i;
  }
}

void testPerceptron(const std::vector<Vector3D>& X, const std::vector<int>& T, const Vector3D& W) {

  int i = 0;
  int success = 0;
  for (xi : X) {
    int u = predict(W, xi);

    if (u == T[i]) {
      ++success;
    }
    ++i;
  }
  std::cout << "Score: " << success << "/" << X.size() << std::endl;
}

void getSamples(std::vector<Vector3D>& X, std::vector<int>& T) {
  int N = 1000;

  for (int i = 0; i < N; ++i) {
    double r = getUnif();
    int ti;
    double xi, yi;
    if (r > 0.5) {
      ti = +1;
    }
    else {
      ti = -1;
    }

    switch (ti) {
      case +1:
        xi = getNorm(+5, +2);
        yi = getNorm(+3, +2);
        break;
      case -1:
        xi = getNorm(+2, +2);
        yi = getNorm(-1, +2); 
        break;
    }
    Vector3D newX {xi, yi, 1};
    X.push_back(newX);
    T.push_back(ti);
  }
}
